from rich.panel import Panel
from rich.table import Table

from tvpy.console import cls
from tvpy.tv_tmdb import load_tvpy
from tvpy.util import all_episodes, existing_episodes, last_episode_to_air


def tv_info(folder):
    try:
        info = load_tvpy(folder)

        episodes = all_episodes(info)
        ss, es = zip(*episodes)
        max_s = max(ss)
        max_e = max(es)
        mat = [[''] * max_e for _ in range(max_s)]

        last_e = last_episode_to_air(info)

        for s, e in episodes:
            mat[s-1][e-1] = '[err]x' if (s, e) <= last_e else ':hourglass:'

        existing = existing_episodes(folder)
        assert not (existing - set(episodes)), f'Unexpected episodes found existing - set(episodes)'
        for s, e in existing_episodes(folder):
            mat[s-1][e-1] = '[success]v'

        def info_line(name, content):
            return f'[bold]{name}:[/bold] {content}'
        info = Panel(
            expand=False,
            title=info["name"],
            renderable='\n'.join([
                info_line('Date', info['first_air_date']),
                info_line('Rating', f'{info["rating"]}'),
                info_line('Genres', f'{", ".join([g["name"] for g in info["genres"]])}'),
                info_line('Overview', info['overview'])
            ]))

        episode_table = Table(
            '',
            title=' [success]v[/success]: existing  [err]x[/err] missing  :hourglass: NA',
            title_style='gray',
            title_justify='left')

        for i in range(len(mat[0])):
            episode_table.add_column(f'{i+1}', justify='center')

        for i, row in enumerate(mat):
            episode_table.add_row(f'{i+1}', *row)

        cls.print(info, episode_table)

    except Exception as e:
        cls.print(f'[err]ERROR:[/err] {e}')



import base64
import json
from datetime import datetime
from io import BytesIO
from pathlib import Path

import requests
from PIL import Image
from rich.status import Status

from tvpy.config import CACHE_DAYS, DATE_FORMAT, POSTER_WIDTH, VERSION
from tvpy.console import cls
from tvpy.tmdb import get, imdb_id, imdb_rating, search_torrent
from tvpy.util import done, load_key, name2title


def img_base64(img):
    buffered = BytesIO()
    img.save(buffered, format="JPEG")
    poster_base64 = base64.b64encode(buffered.getvalue()).decode('utf-8')
    return poster_base64


def resize_poster(img, width=POSTER_WIDTH):
    w, h = img.size
    img = img.resize((width, int(width / w * h)))
    return img


def get_img(poster_path):
    response = requests.get(poster_path)
    img = Image.open(BytesIO(response.content))
    return img


def load_tvpy(folder, tries=1):
    try:
        folder = Path(folder)
        folder.mkdir(
            parents=True,
            exist_ok=True)

        tvpy_json = folder / '.tvpy.json'
        with open(tvpy_json, 'r') as f:
            tvpy = json.load(f)

        assert tvpy['version'] == VERSION

        delta = datetime.now() - \
            datetime.strptime(tvpy['uptodate'], DATE_FORMAT)
        assert delta.days <= CACHE_DAYS

        return tvpy
    except:
        if tries <= 0:
            raise FileNotFoundError(f'Can not load or download {tv_tmdb}')

        tv_tmdb(folder)
        return load_tvpy(folder, tries - 1)


def tv_tmdb(folder, force=False):
    folder = Path(folder)
    folder.mkdir(
        parents=True,
        exist_ok=True)

    key = load_key()
    tvpy_json = folder / '.tvpy.json'

    with Status('', console=cls) as status:
        try:
            assert not force
            load_tvpy(folder)
        except:
            status.update('[info]Searching TMDB...')
            query = name2title(folder.name)
            res = search_torrent(key, query)

            if res is None:
                status.stop()
                cls.print(f'[err]Error:[/err] Could not find info for {query}')
                return

            tmdb_id = res['id']
            poster = get_img(res['poster_path'])
            poster = resize_poster(poster)
            poster.save(folder / '.poster.jpg')

            iid = imdb_id(key, tmdb_id)

            res = get(key, tmdb_id)
            res = {**res, **imdb_rating(iid)}
            res['version'] = VERSION
            res['uptodate'] = datetime.now().strftime(DATE_FORMAT)
            res['imdb_id'] = iid
            # res['poster_base64'] = img_base64(poster)
            with open(tvpy_json, 'w') as out:
                json.dump(res, out)

    done()



from importlib import resources

from tvpy.tv_tmdb import load_tvpy


def tv_html(folder, out_html='index.html'):
    import tvpy

    css = resources.read_text(tvpy, 'index.css')
    js = resources.read_text(tvpy, 'index.js')
    html = resources.read_text(tvpy, 'index.html')

    data = [load_tvpy(folder)]

    html = html.format(data=data, js=js, css=css)

    with open(out_html, 'w') as h:
        h.write(html)



from rich.console import Console
from rich.style import Style
from rich.theme import Theme

info = Style.parse('dim cyan')
warn = Style.parse('orange1')
err = Style.parse('bold red')
success = Style.parse('green')

cls = Console(
    record=True,
    theme=Theme({
        'info': info,
        'warn': warn,
        'err': err,
        'success': success}))



import time

from humanize import naturaldelta

from tvpy.console import cls
from tvpy.tv_clean import tv_clean
from tvpy.tv_download import tv_download
from tvpy.tv_follow import read_follow, tv_follow, tv_unfollow
from tvpy.tv_info import tv_info
from tvpy.tv_rename import tv_rename
from tvpy.tv_subs import tv_subs
from tvpy.tv_tmdb import tv_tmdb

logo = (r'''
📺📺📺📺📺📺📺📺📺📺📺📺📺📺
[yellow2]  _______    _____       
[yellow2] |__   __|  |  __ \      
[yellow2]    | |_   _| |__) |   _ 
[yellow2]    | \ \ / /  ___/ | | |
[yellow2]    | |\ V /| |   | |_| |
[yellow2]    |_| \_/ |_|    \__, |
[yellow2]                    __/ |
[yellow2]                   |___/ 

🥧🥧🥧🥧🥧🥧🥧🥧🥧🥧🥧🥧🥧                                                        
''')


def tvpy(folder=None,  k=3):
    cls.print(logo)

    if folder is None:
        folders = read_follow()
    else:
        tv_follow(folder)
        folders = [folder]

    for folder in folders:
        err_msg = f'[red]Something went wrong with {folder}. Unfollowing...'
        try:
            tvpy_folder(folder, k)
        except:
            cls.print(err_msg)
            tv_unfollow(folder)


def tvpy_folder(folder, k):
    def sep(title):
        cls.print()
        cls.print(f'[dim orchid1]{title}')
    try:
        sep('Generating .tvpy.json')
        tv_tmdb(folder)

        sep('Downloading episodes')
        tv_download(folder, k=k, raise_ki=True)

        sep('Renaming files')
        tv_rename(folder)

        sep('Downloading subtitles')
        tv_subs(folder)

        sep('Removing unused files')
        tv_clean(folder)

        tv_info(folder)

    except KeyboardInterrupt:
        cls.print(f'[warn]Aborting {folder}')



from pathlib import Path

from PTN import parse
from rich.status import Status

from tvpy.addic7ed import Addic7ed
from tvpy.config import get_lang
from tvpy.console import cls
from tvpy.tv_tmdb import load_tvpy
from tvpy.util import done, existing_episodes, files_subs, title2file_name
from tvpy.wizdom import Wizdom


def existing_subs(folder):
    res = [parse(f.name) for f in files_subs(folder)]
    return {(e['season'], e['episode']) for e in res}


def tv_subs(folder):
    missing_subs = existing_episodes(folder) - existing_subs(folder)
    if not missing_subs:
        done()
        return

    info = load_tvpy(folder)
    title = info['name']
    lang = get_lang()
    provider = Wizdom() if lang == 'Hebrew' else Addic7ed()

    with Status(f'[info]Downloading subtitles...', console=cls) as status:
        srts = provider.get_subs(
            imdb_id=info['imdb_id'],
            query=title,
            lang=lang,
            episodes=missing_subs)

        for s, e, srt in srts:
            path = Path(folder) / f'{title2file_name(title, s, e)}.srt'
            cls.print(f':clapper: [success]{path}')
            with open(path, 'wb') as f:
                f.write(srt)

    done()



from abc import ABC, abstractmethod
from typing import Iterable, Set, Tuple


class SubProvider(ABC):
    @abstractmethod
    def get_subs(self, *, imdb_id=None, query=None, lang=None, episodes: Set[Tuple[int, int]] = ...) -> Iterable[Tuple[int, int, str]]:
        raise NotImplementedError()



from pathlib import Path

from tvpy.config import default, load_config, save_config
from tvpy.util import title2folder_name


def read_follow():
    config = load_config()
    return config[default.follow.name]


def tv_unfollow(folder):
    path = str(Path(folder).absolute())
    config = load_config()
    follows = set(config[default.follow.name]) - {path}
    config[default.follow.name] = follows
    save_config(config)


def tv_follow(folder):
    path = str(Path(folder).absolute())
    config = load_config()
    follows = list(set(config[default.follow.name] + [path]))
    config[default.follow.name] = follows
    save_config(config)



from enum import Enum
from pathlib import Path

import toml
from simple_term_menu import TerminalMenu

from tvpy.console import cls

VERSION = 0.4
POSTER_WIDTH = 160
DATE_FORMAT = r'%Y-%m-%d'
CACHE_DAYS = 1


LANGS = [
    'Arabic',
    'Bulgarian',
    'Croatian',
    'Czech',
    'Dutch',
    'English',
    'French',
    'German',
    'Greek',
    'Hebrew',
    'Hungarian',
    'Italian',
    'Norwegian',
    'Persian',
    'Portuguese',
    'Romanian',
    'Russian',
    # 'Spanish',
    'Swedish',
    'Turkish']


class default(Enum):
    lang = ''
    follow = []


def save_config(config):
    with open(Path.home() / '.tvpy.toml', 'w') as f:
        toml.dump(config, f)


def init_config():
    save_config({
        default.lang.name: '',
        default.follow.name: []})


def validate(config):
    for key in default:
        if key.name not in config:
            config[key.name] = key.value

    return config


def load_config():
    home = Path.home()
    tvpy_toml = home / '.tvpy.toml'
    if not tvpy_toml.exists():
        init_config()

    with open(tvpy_toml, 'r') as f:
        config = validate(toml.load(f))
        lang = config[default.lang.name]
        if lang not in LANGS:
            cls.print('[info]Please select the subtitle language:')
            i = TerminalMenu(LANGS).show()
            assert type(i) is int
            config[default.lang.name] = LANGS[i]
            save_config(config)

        return config


def get_lang():
    config = load_config()
    return config[default.lang.name]



import shutil
from pathlib import Path

from PTN import parse

from tvpy.console import cls
from tvpy.util import done

keep_types = {'MKV', 'MP4', 'AVI', 'SRT'}


def must_keep(p: Path):
    if p.is_dir():
        return not can_remove(p)

    info = parse(p.name)

    return (
        'season' in info
        and 'episode' in info
        and 'filetype' in info
        and info['filetype'] in keep_types)


def can_remove(p: Path):
    info = parse(p.name)

    if 'season' not in info or 'episode' not in info:
        return False

    if p.is_dir():
        return not any(must_keep(f) for f in p.iterdir())

    if not 'filetype' in info:
        return True

    return info['filetype'] not in keep_types


def tv_clean(folder):
    folder = Path(folder)
    for f in folder.iterdir():
        if can_remove(f):
            cls.print(f':wastebasket: [warn]{f.name}')
            if f.is_dir():
                shutil.rmtree(f)
            else:
                f.unlink()

    done()




from pathlib import Path

import PTN

from tvpy.console import cls
from tvpy.tv_tmdb import load_tvpy
from tvpy.util import done, files_r, title2file_name


def tv_rename(folder):
    folder = Path(folder)
    tvpy = load_tvpy(folder)
    for file in files_r(folder):
        try:
            info = PTN.parse(file.name)
            name = f'{title2file_name(tvpy["name"], info["season"], info["episode"])}{file.suffix.lower()}'
            if name != file.name:
                file.rename(folder / name)
                cls.print(f':keyboard: [success]{name}')
        except:
            pass

    done()



import re
from itertools import chain
from pathlib import Path

from PTN import parse

from tvpy.console import cls


def file_size_in_mb(size_str: str):
    # e.g 1.8 GB, 222.8 MB
    size, units = size_str.split(' ')
    assert units in {'GB', 'MB'}
    size = float(size)
    return size if units == 'MB' else size * 2**10


def load_key():
    key_txt = Path('key.txt')
    if key_txt.exists():
        with open(key_txt, 'r') as f:
            return f.read().strip()

    return '7bfa2260d938bb3881e0dd89c47a6021'


def title2file_name(title, s, e):
    name = re.sub(r"[']", '', title)
    name = re.sub(r'[^a-zA-Z0-9]', ' ', name)
    name = re.sub(r' +', '.', name)

    return f'{name}.S{s:02}E{e:02}'


def title2folder_name(title):
    name = re.sub(r'[^a-zA-Z0-9]', ' ', title)
    name = re.sub(r' +', '.', name)

    return name


def name2title(name):
    return Path(name).name.replace('.', ' ').replace('_', ' ')


def folders(root):
    return [x for x in Path(root).iterdir() if x.is_dir()]


def tvpys(folder):
    return Path(folder).rglob('.tvpy.json')


def files(folder, patterns=['*.mkv', '*.avi', '*.mp4', '*.srt']):
    return chain(*[Path(folder).glob(e) for e in patterns])


def files_r(folder, patterns=['*.mkv', '*.avi', '*.mp4', '*.srt']):
    return chain(*[Path(folder).rglob(e) for e in patterns])


def files_media(root):
    return files(root, patterns=['*.mkv', '*.avi', '*.mp4'])


def files_subs(root):
    return files(root, patterns=['*.srt'])


def files_media_r(root):
    return files_r(root, patterns=['*.mkv', '*.avi', '*.mp4'])


def files_subs_r(root):
    return files_r(root, patterns=['*.srt'])


def existing_episodes(folder):
    res = [parse(f.name) for f in files_media(folder)]
    return {(e['season'], e['episode']) for e in res if 'season' in e}


def all_episodes(info):
    episodes = {
        (s['season_number'], i)
        for s in info['seasons']
        for i in range(1, s['episode_count'] + 1)}

    return sorted({(s, e) for s, e in episodes if s > 0})


def last_episode_to_air(info):
    res = info['last_episode_to_air']
    return (res['season_number'], res['episode_number'])


def on_air_episodes(info):
    return {
        e for e in all_episodes(info)
        if e <= last_episode_to_air(info)}


def missing_episodes(folder, tvpy, k):
    episodes = sorted(on_air_episodes(tvpy) - existing_episodes(folder))
    return episodes[:k]


def done():
    cls.print(f'[green]Done :+1:')



import io
import zipfile

import requests

from tvpy.subs import SubProvider

# def down_sub(sub_id, out_zip):
#     url = f'https://wizdom.xyz/api/files/sub/{sub_id}'
#     with open(out_zip, 'wb') as out:
#         out.write(requests.get(url).content)


def get_subs_json(imdb_id):
    url = f'https://wizdom.xyz/api/releases/{imdb_id}'
    res = requests.get(url).json()
    return res['subs']


def select_sub_ids(*, subs, episodes):
    for s, e in episodes:
        try:
            yield s, e, subs[f'{s}'][f'{e}'][0]['id']
        except:
            pass


def download_zips(sub_ids):
    for s, e, id in sub_ids:
        url = f'https://wizdom.xyz/api/files/sub/{id}'
        yield s, e, requests.get(url).content


def srts_from_zips(zips):
    for s, e, zip in zips:
        with zipfile.ZipFile(io.BytesIO(zip)) as z:
            srts = [f for f in z.infolist() if f.filename[-4:] == '.srt']
            for srt in srts:
                with z.open(srt, 'r') as content:
                    yield s, e, content.read()


class Wizdom(SubProvider):
    def get_subs(self, *, imdb_id=None, query=None, lang=None, episodes=...):
        assert lang == 'Hebrew'
        sub_ids = select_sub_ids(
            subs=get_subs_json(imdb_id),
            episodes=episodes)

        zips = download_zips(sub_ids)
        yield from srts_from_zips(zips)



from fire import Fire


def tv_follow():
    from tvpy.tv_follow import tv_follow
    Fire(tv_follow)


def tv_tmdb():
    from tvpy.tv_tmdb import tv_tmdb
    Fire(tv_tmdb)


def tv_download():
    from tvpy.tv_download import tv_download
    Fire(tv_download)


def tv_subs():
    from tvpy.tv_subs import tv_subs
    Fire(tv_subs)


def tv_rename():
    from tvpy.tv_rename import tv_rename
    Fire(tv_rename)


def tv_clean():
    from tvpy.tv_clean import tv_clean
    Fire(tv_clean)


def tv_info():
    from tvpy.tv_info import tv_info
    Fire(tv_info)


def tv_html():
    from tvpy.tv_html import tv_html
    Fire(tv_html)


def tvpy():
    from tvpy.tvpy import tvpy
    Fire(tvpy)


if __name__ == '__main__':
    import json

    from py1337x import py1337x

    with open('tmp.json', 'w') as f:

        json.dump(py1337x().search('Andor'), f)



from collections import defaultdict

import requests
from lxml import etree, html

from tvpy.config import LANGS
from tvpy.subs import SubProvider

HEADERS = {
    'referer': 'https://www.addic7ed.com/',
    'sec-fetch-mode': 'navigate'}


def show_id(name):
    url = f'https://www.addic7ed.com/search.php?search={name}&Submit=Search'
    return requests.get(url, allow_redirects=True).url.split('/')[-1]


def subs(show_id, season):
    res = requests.get(f'https://www.addic7ed.com/ajax_loadShow.php?show={show_id}&season={season}')
    dom = html.fromstring(res.content)
    for tr in dom.cssselect('tr.epeven.completed'):
        tree = etree.ElementTree(tr)
        path = tree.getpath(tr)
        season, episode, lang, *_ = tr.xpath(f'{path}/td/text()')
        link = tr.xpath(f'{path}//a/@href')[-1]
        yield int(season), int(episode), lang, link


def filter_subs(*, subs, lang, episodes):
    episodes = set(episodes)
    for s, e, l, link in subs:
        if (s, e) in episodes and l.lower() == lang.lower():
            episodes.remove((s, e))
            yield s, e, link


class Addic7ed(SubProvider):
    def get_subs(self, *, imdb_id=None, query=None, lang=..., episodes=...):
        assert lang in LANGS

        id = show_id(query)
        se = defaultdict(set)
        for s, e in episodes:
            se[s].add(e)

        for season in se.keys():
            for s, e, link in filter_subs(
                    subs=subs(id, season),
                    lang=lang,
                    episodes=episodes):

                yield s, e, requests.get(
                    f'https://www.addic7ed.com{link}',
                    headers=HEADERS).content



import math
import random
import urllib.parse
from dataclasses import dataclass
from pathlib import Path


def get_name(magnet_uri):
    query = urllib.parse.urlparse(magnet_uri).query
    query = urllib.parse.parse_qs(query)

    return query['dn'][0]


@dataclass
class Status:
    progress: float
    total_done: float
    download_rate: float
    upload_rate: float
    num_peers: int


def rnd_file_size_mb():
    return random.randint(200, 300) * 1_048_576


class Handler:
    def __init__(self):
        self.file_size_mb = rnd_file_size_mb()
        self.time = 0
        self.progress = 0
        self.num_peers = 0

    def status(self):
        self.time += 1
        self.num_peers += random.randint(-2, 5)
        self.num_peers = max(1, self.num_peers)
        self.progress += random.random() / 10 * math.log(self.num_peers)
        self.progress = min(1, self.progress)

        total_done = self.file_size_mb * self.progress
        download_rate = total_done / self.time

        return Status(
            progress=self.progress,
            total_done=total_done,
            download_rate=download_rate,
            upload_rate=download_rate / random.randint(8, 12),
            num_peers=self.num_peers)


@dataclass
class Params:
    name: str
    save_path: str = ''


class Session:
    def add_torrent(self, params: Params):
        path = Path(params.save_path) / params.name
        path.mkdir(exist_ok=True)
        path = path / f'{params.name}.mkv'
        path.touch(exist_ok=True)
        return Handler()


class dummy:
    def session(self):
        return Session()

    def parse_magnet_uri(self, link):
        return Params(name=get_name(link))


lt = dummy()

if __name__ == '__main__':
    h = Handler()
    print(h.time)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tvpy']

package_data = \
{'': ['*']}

install_requires = \
['1337x>=1.2.3,<2.0.0',
 'Pillow>=9.2.0,<10.0.0',
 'cssselect>=1.2.0,<2.0.0',
 'fire>=0.4.0,<0.5.0',
 'humanize>=4.4.0,<5.0.0',
 'libtorrent>=2.0.7,<3.0.0',
 'lxml>=4.9.1,<5.0.0',
 'parse-torrent-title>=2.4,<3.0',
 'pyright>=1.1.273,<2.0.0',
 'rich>=12.5.1,<13.0.0',
 'simple-term-menu>=1.5.2,<2.0.0',
 'toml>=0.10.2,<0.11.0',
 'tqdm>=4.64.0,<5.0.0']

entry_points = \
{'console_scripts': ['tv-clean = tvpy.main:tv_clean',
                     'tv-download = tvpy.main:tv_download',
                     'tv-follow = tvpy.main:tv_follow',
                     'tv-info = tvpy.main:tv_info',
                     'tv-rename = tvpy.main:tv_rename',
                     'tv-subs = tvpy.main:tv_subs',
                     'tv-tmdb = tvpy.main:tv_tmdb',
                     'tvpy = tvpy.main:tvpy']}

setup_kwargs = {
    'name': 'tvpy',
    'version': '0.0.39',
    'description': '📺 TvPy',
    'long_description': '# 📺📺 TvPy 🥧🥧\n\nManage TV shows from the terminal.\n\n![demo](https://github.com/gkutiel/tvpy/raw/1a6686c863a74e1c8dd66b4f9087de2f8a349c61/demo.gif)\n\n## Installation\n```shell\n> pip install tvpy\n```\n\n## Usage\n```shell\n> mkdir The.Peripheral \n> tvpy The.Peripheral \n```\n\n## Other commands\nDownload information from TMDB:\n```shell\n> mkdir The.Peripheral \n> tv-json The.Peripheral\n```\n\nDisplay information about a tv show:\n```shell\n> mkdir The.Peripheral \n> tv-info The.Peripheral\n```\n\nDownload a tv show:\n```shell\n> mkdir The.Peripheral \n> tv-download The.Peripheral\n```\n\nDownload subtitles for a tv show:\n```shell\n> mkdir The.Peripheral \n> tv-subs The.Peripheral\n```\n\nRename files to match the pattern `<title>.S<season>E<episode>.<ext>`\n```shell\n> mkdir The.Peripheral \n> tv-rename The.Peripheral\n```\n\n| ⚠️ Danger |\n|----------|\nRemove unused files\n```shell\n> mkdir The.Peripheral \n> tv-clean The.Peripheral\n```\n\n## Configuration\nA small `.tvpy.toml` configuration file is located at your home directory.\nIts content looks like that:\n```toml\nlang = "English"\nfollow = []\n```\n`lang` determines the subtitle language.\n`follow` is a list of TV shows you are watching.',
    'author': 'Gilad Kutiel',
    'author_email': 'gilad.kutiel@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/gkutiel/tvpy/tree/master',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import json
import re
import urllib.parse

import requests


def imdb_id(key, id):
    res = requests.get(
        f'https://api.themoviedb.org/3/tv/{id}?api_key={key}&append_to_response=external_ids')
    res = res.json()
    return res['external_ids']['imdb_id']


def imdb_rating(id):
    url = f'https://p.media-imdb.com/static-content/documents/v1/title/{id}/ratings%3Fjsonp=imdb.rating.run:imdb.api.title.ratings/data.json?u=null&s=p3'
    res = requests.get(url).text
    res = re.search(r'\{.*\}', res)
    assert res is not None, f'Failed to fetch rating for {id} at {url}'
    res = json.loads(res.group(0))
    resource = res['resource']
    return {k: resource[k] for k in ('rating', 'ratingCount')}


def search_torrent(key, q):
    try:
        q = urllib.parse.quote_plus(q)
        url = f'https://api.themoviedb.org/3/search/tv?api_key={key}&query={q}'
        res = requests.get(url)
        res = res.json()
        res = res['results'][0]

        res['poster_path'] = f'https://image.tmdb.org/t/p/original/{res["poster_path"]}'

        return res
    except Exception as e:
        return None


def get(key, tv_id):
    url = f'https://api.themoviedb.org/3/tv/{tv_id}?api_key={key}'
    res = requests.get(url)
    res = res.json()

    return res



from pprint import pprint
from pathlib import Path
from time import sleep
from typing import List, Tuple

import libtorrent as lt
import rich.status
from py1337x import py1337x
from rich.progress import (BarColumn, Progress, TaskID, TaskProgressColumn,
                           TextColumn, TimeRemainingColumn)

from tvpy.console import cls
from tvpy.lt import Handler
from tvpy.tv_tmdb import load_tvpy
from tvpy.util import (done, file_size_in_mb, missing_episodes, name2title,
                       title2file_name)


def q(folder, season: int, episode: int):
    query = name2title(Path(folder).name)

    return f'{query} S{season:02}E{episode:02}'


def down(magnets, down_folder, raise_ki):
    s = lt.session()
    try:
        with Progress(
                TextColumn("[progress.description]{task.description}"),
                BarColumn(),
                TaskProgressColumn(),
                TimeRemainingColumn(),
                TextColumn('[dim cyan]Size: {task.fields[size]}MB'),
                TextColumn('[green]Down: {task.fields[down]}kB/s'),
                TextColumn('[blue]Up: {task.fields[up]}kB/s'),
                TextColumn('[orange1]Peers: {task.fields[peers]}')) as progress:

            tasks: List[Tuple[Handler, TaskID]] = []
            for name, link in magnets:
                params = lt.parse_magnet_uri(link)
                params.save_path = str(down_folder)

                h = s.add_torrent(params)
                task = progress.add_task(
                    f'[green]{name}', total=1, size='', down='', up='', peers='')
                tasks.append((h, task))

            while not progress.finished:
                for h, task in tasks:
                    status = h.status()
                    progress.update(
                        task,
                        completed=status.progress,
                        size=f'{status.total_done / 1048576:.1f}',
                        down=f'{status.download_rate / 1000:.1f}',
                        up=f'{status.upload_rate / 1000:.1f}',
                        peers=status.num_peers)

                sleep(1)
    except KeyboardInterrupt:
        if raise_ki:
            raise KeyboardInterrupt()


def search_torrent(query):
    torrents = py1337x()
    res = torrents.search(query)

    items = res['items']

    if items is None:
        return None

    items = sorted(
        items,
        key=lambda item: int(item['seeders']),
        reverse=True)

    link = items[0]['link']
    link = link.replace('1377x', '1337x')
    return torrents.info(link)


def tv_download(folder, k=10, raise_ki=False):
    tvpy = load_tvpy(folder)

    magnets = []
    with rich.status.Status('', console=cls) as status:
        for s, e in missing_episodes(folder, tvpy, k=k):
            name = title2file_name(tvpy['name'], s, e)
            magnet = Path(folder) / f'{name}.magnet'
            if magnet.exists():
                with open(magnet, 'r') as f:
                    magnet_link = f.read()
            else:
                query = q(folder, s, e)
                status.update(f'[info]Searching for {query}')
                info = search_torrent(query)
                if not info:
                    cls.print(f'[warn]Could not find torrent for {query}')
                    continue

                magnet_link = info['magnetLink']

                with open(magnet, 'w') as f:
                    f.write(magnet_link)

            magnets.append((name, magnet_link))

    if magnets:
        down(magnets, folder, raise_ki)

    done()
